@@ -1,15 +1,16 @@
 #!/usr/bin/env python3
 """Example of user-defined Lite Data Objects"""
-__version__ = '1.0.6 2021-11-19'# no_float32 removed 
- 
+__version__ = '2.0.0 2023-02-27'#
+
 import sys, time, threading
 from timeit import default_timer as timer
 import numpy as np
 
-try:
-    from liteserv import liteserver
-except:
+if True:#try:
     from liteserver import liteserver
+else:#except Exception as e:
+    print(f'ERROR: Could not import liteserv.liteserver: {e}')
+    #from liteserver import liteserver
 LDO = liteserver.LDO
 Device = liteserver.Device
 
@@ -50,15 +51,17 @@ class Scaler(Device):
           'time':       LDO('R','Current time',0., getter=self.get_time),
           'number':     LDO('RWE','Test number',0., opLimits=(-1000000,1000000)\
           ,setter=self.set_number),
+          'multicurve': LDO('R','Array of 8 of noise curves',
+                        np.zeros(8*100).reshape(8,100).astype('uint16')),
           'text':       LDO('RWE','Test text', ['Test'], setter=self.set_text),
           'cycle':      LDO('R','Cycle number',0),
           'rps':        LDO('R','Cycles per second',0.,units='Hz'),
           'publishingSpeed': LDO('R', 'Instanteneous publishing speed of published data', 0., units='MB/s'),
           'dataSize':   LDO('R', 'Size of published data', 0., units='KB'),
           'chunks':     LDO('R', 'Number of chunks in UDP transfer, for lowest latency it should be 1', 0.),
-          'udpSpeed':   LDO('R', 'Instanteneous socket.send spped', 0., units='MB/s'),
+          'udpSpeed':   LDO('R', 'Instanteneous socket.send speed', 0., units='MB/s'),
         }
-        super().__init__(name,pars)
+        super().__init__(name, pars)
         self.start()
     #``````````````Overridables```````````````````````````````````````````````        
     def start(self):
@@ -68,74 +71,95 @@ class Scaler(Device):
         thread.start()
     #,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
 
+    def update_multicurve(self, timestamp):
+        pv_multicurve = self.PV['multicurve']
+        shape = pv_multicurve.value.shape
+        #print(f'multicurve shape: {shape}')
+        #breakpoint()
+        for i in range(shape[0]):
+            pv_multicurve.value[i] = (np.random.random(shape[1])*100 + i*100).astype('uint16')
+        pv_multicurve.timestamp = timestamp
+
     def reset(self):
         print('resetting scalers of %s'%self.name)
-        for i in range(len(self.counters.value)):
-            self.counters.value[i] = 0
+        pv_counters = self.PV['counters']
+        for i in range(len(pv_counters.value)):
+            pv_counters.value[i] = 0
         t = time.time()
-        self.counters.timestamp = t
-        self.reset.timestamp = t
+        pv_counters.timestamp = t
+        self.PV['reset'].timestamp = t
 
     def set_number(self):
-        print('Setting number to '+str(self.number.value))
+        print('Setting number to '+str(self.PV['number'].value))
 
     def set_text(self):
-        msg = 'Setting text to '+str(self.text.value)
+        msg = 'Setting text to '+str(self.PV['text'].value)
         print(msg)
-        raise ValueError(msg)
+        #raise ValueError(msg)
 
     def get_time(self):
-        self.time.value = time.time()
+        self.PV['time'].value = time.time()
 
     def _state_machine(self):
         time.sleep(.2)# give time for server to startup
-
-        self.cycle.value = 0
+        pv_status = self.PV['status']
+        pv_cycle = self.PV['cycle']
+        pv_cycle.value = 0
         prevCycle = 0
-        ns = len(self.counters.value)
+        pv_counters = self.PV['counters']
+        pv_frequency = self.PV['frequency']
+        pv_increments = self.PV['increments']
+        pv_image = self.PV['image']
+        pv_run = self.PV['run']
+        pv_publishingSpeed = self.PV['publishingSpeed']
+        pv_chunks = self.PV['chunks']
+        pv_dataSize = self.PV['dataSize']
+        pv_udpSpeed = self.PV['udpSpeed']
+        ns = len(pv_counters.value)
         timestamp = time.time()
         periodic_update = time.time()
         maxChanks = 0
         while not Device.EventExit.is_set():
             #TODO: with liteserver-v76 we have to use value[0]
-            if self.run.value[0][:4] == 'Stop':
+            if pv_run.value[0][:4] == 'Stop':
                 break
-            #waitTime = 1./self.frequency.value - (time.time() - timestamp)
-            waitTime = 1./self.frequency.value[0] - (time.time() - timestamp)
+            #waitTime = 1./pv_frequency.value - (time.time() - timestamp)
+            waitTime = 1./pv_frequency.value[0] - (time.time() - timestamp)
             Device.EventExit.wait(waitTime)
             timestamp = time.time()
             dt = timestamp - periodic_update
             if dt > 10.:
                 periodic_update = timestamp
                 if server.Dbg > 0:
-                    printi(f'cycle of {self.name}:{self.cycle.value}, wt:{round(waitTime,4)}')
+                    printi(f'cycle of {self.name}:{pv_cycle.value}, wt:{round(waitTime,4)}')
                 #print(f'periodic update: {dt}')
                 if maxChanks > 1:
                     msg = 'WARNING: published data are chopped, latency will increase'
                     maxChanks = 0
                 else:
                     msg = f'periodic update {self.name} @{round(timestamp,3)}'
-                self.status.value = msg
-                self.status.timestamp = timestamp
-                #print(self.status.value[0])
-                #Device.setServerStatusText('Cycle %i on '%self.cycle.value[0]+self.name)
-                self.rps.value = (self.cycle.value - prevCycle)/dt
-                self.rps.timestamp = timestamp
-                prevCycle = self.cycle.value
+                pv_status.set_valueAndTimestamp(msg, timestamp)
+                #print(pv_status.value[0])
+                #Device.setServerStatusText('Cycle %i on '%pv_cycle.value[0]+self.name)
+                self.PV['rps'].set_valueAndTimestamp(\
+                  (pv_cycle.value - prevCycle)/dt, timestamp)
+                prevCycle = pv_cycle.value
             # increment counters individually
-            for i,increment in enumerate(self.increments.value[:ns]):
-                #print(instance+': c,i='+str((self.counters.value[i],increment)))
-                self.counters.value[i] += increment
+            for i,increment in enumerate(pv_increments.value[:ns]):
+                #print(instance+': c,i='+str((pv_counters.value[i],increment)))
+                pv_counters.value[i] += increment
                 
             # increment pixels in the image
             # this is very time consuming:
-            #self.image.value[0] = (self.image.value[0] + 1).astype('uint8')
+            #pv_image.value[0] = (pv_image.value[0] + 1).astype('uint8')
 
             # change only one pixel            
-            self.image.value[0,0,0] = np.uint8(self.cycle.value)
+            pv_image.value[0,0,0] = np.uint8(pv_cycle.value)
      
-            self.cycle.value += 1
+            pv_cycle.value += 1
 
+            self.update_multicurve(timestamp)
+                
             #print('publish all modified parameters of '+self.name)
             try:
                 dt = server.Perf['Seconds'] - self._prevs[1]
@@ -143,12 +167,12 @@ class Scaler(Device):
             except:
                 mbps = 0.
             self._prevs = server.Perf['MBytes'],server.Perf['Seconds']
-            self.udpSpeed.value = mbps
+            pv_udpSpeed.value = mbps
 
             # invalidate timestamps for changing variables, otherwise the
             # publish() will ignore them
-            for i in [self.counters, self.image, self.cycle, self.udpSpeed,
-                self.publishingSpeed, self.dataSize, self.chunks, self.time]:
+            for i in [pv_counters, pv_image, pv_cycle, pv_udpSpeed,
+                pv_publishingSpeed, pv_dataSize, pv_chunks, self.PV['time']]:
                 i.timestamp = timestamp
 
             ts = timer()
@@ -157,11 +181,10 @@ class Scaler(Device):
             if shippedBytes:
                 ss = round(shippedBytes / (timer() - ts) / 1.e6, 3)
                 #print(f'sb: {shippedBytes}')            
-                self.publishingSpeed.value = ss
-                #printd(f'publishing speed of {self.name}: {ss}')
-                self.dataSize.value = round(shippedBytes/1000.,1)
-                self.chunks.value = (shippedBytes-1)//liteserver.ChunkSize + 1
-                maxChanks = max(maxChanks, self.chunks.value)
+                pv_publishingSpeed.value = ss
+                pv_dataSize.value = round(shippedBytes/1000.,1)
+                pv_chunks.value = (shippedBytes-1)//liteserver.ChunkSize + 1
+                maxChanks = max(maxChanks, pv_chunks.value)
         print('Scaler '+self.name+' exit')
 #,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,S,,,,,,,,,,,,,,,,,,,,,,,,,,,,
 # parse arguments
