@@ -1,10 +1,6 @@
 #!/usr/bin/env python3
 """LiteServer for USB cameras"""
-#__version__ = 'v01 2019-06-03'# created
-#__version__ = 'v02 2019-06-10'# using latest liteServer
-#__version__ = 'v03 2021-04-21'# pause parameter not needed, timestamping before publishing is not necessary
-#__version__ = 'v04 2021-09-21'# argparse
-__version__ = '1.0.6 2021-19-21'#--verbose, do not call aborted()
+__version__ = '2.0.0 2023-03-23'# compatible with liteserver 2.x.x
 
 #TODO: sometimes it does not start/stop nicely, Action required: disconnect camera, then run guvcview
 
@@ -23,7 +19,6 @@ from liteserver import liteserver
 LDO = liteserver.LDO
 Device = liteserver.Device
 EventExit = Device.EventExit
-printd = liteserver.printd
 
 #````````````````````````````Helper functions`````````````````````````````````
 def printw(msg): print('WARNING: '+msg)
@@ -79,24 +74,24 @@ class Camera(Device):
         
     def _state_machine(self):
         while not self.EventExit.is_set():
-            EventExit.wait(self.sleep.value[0])
+            EventExit.wait(self.PV['sleep'].value[0])
             ret, img = self._cv2_cap.read()
             if not ret:
                 printw("Error reading image")
                 continue
             timestamp = time.time()
             printd(f'img.shape {img.shape}{img.dtype}, data: {str(img)[:200]}...\n')
-            if self.shape.value[0] == 0:
-                self.shape.value = img.shape
-                self.shape.timestamp = timestamp
-            self.image.value = img
-            if self.subscribe.value[0] == 'On':
-                self.image.timestamp = timestamp
-            self.count.value[0] += 1
-            self.count.timestamp = timestamp
+            if self.PV['shape'].value[0] == 0:
+                self.PV['shape'].value = img.shape
+                self.PV['shape'].timestamp = timestamp
+            self.PV['image'].value = img
+            if self.PV['subscribe'].value[0] == 'On':
+                self.PV['image'].timestamp = timestamp
+            self.PV['count'].value[0] += 1
+            self.PV['count'].timestamp = timestamp
             #msg=f'Ready to publish@{timestamp}'
-            #self.status.value[0] = msg
-            #self.status.timestamp = timestamp
+            #self.PV['status'].value[0] = msg
+            #self.PV['status'].timestamp = timestamp
             shippedBytes = self.publish()
             printd(f'shippedBytes: {shippedBytes}')
 
